#! /usr/bin/env python
# -*- coding: iso-8859-1 -*-

## Copyright 2000-2004 by LivingLogic AG, Bayreuth/Germany.
## Copyright 2000-2004 by Walter Drwald
##
## All Rights Reserved
##
## Permission to use, copy, modify, and distribute this software and its documentation
## for any purpose and without fee is hereby granted, provided that the above copyright
## notice appears in all copies and that both that copyright notice and this permission
## notice appear in supporting documentation, and that the name of LivingLogic AG or
## the author not be used in advertising or publicity pertaining to distribution of the
## software without specific, written prior permission.
##
## LIVINGLOGIC AG AND THE AUTHOR DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
## INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
## LIVINGLOGIC AG OR THE AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
## DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
## IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR
## IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.


r"""
<par><module>ANSIStyle</module> is a module that adds &ansi; color capability
to an output stream.</par>

<par>This is done by wrapping an output stream in a <class>Stream</class> object:</par>
<prog>
import sys
from ll import ansistyle

s = ansistyle.Stream(sys.stdout)
</prog>

<par>In addition to the <method>write</method> method that writes to the underlying
stream, <class>ansistyle.Stream</class> has three methods for handling color.
<method>pushcolor</method> takes an integer argument between 0 and 127. The values 0-127
are equivalent to the following escape sequences:</par>
<ulist>
<item>color 0: <lit>"\033[30m"</lit></item>
<item>...</item>
<item>color 7: <lit>"\033[37m"</lit></item>
<item>color 8: <lit>"\033[1;30m"</lit></item>
<item>...</item>
<item>color 15: <lit>"\033[1;37m"</lit></item>
<item>color 16: <lit>"\033[40m"</lit></item>
<item>...</item>
<item>color 31: <lit>"\033[40;1;37m"</lit></item>
<item>color 32: <lit>"\033[41;30m"</lit></item>
<item>...</item>
<item>color 127: <lit>"\033[47;1;37m"</lit></item>
</ulist>
<par>The simplest way to specify a color is in hexadecimal, the high nybble
is the background color, the low nybble the foreground color.</par>

<par>After calling <method>pushcolor</method> you can write to the stream with
<method>write</method>, and the output will appear in that color. Returning
to the previously active color is possible with <method>popcolor</method>:</par>
<prog>
s.write("spam")
s.pushcolor(0x7d)
s.write("eggs")
s.popcolor()
</prog>
<par><method>popcolor</method> returns the color that was active before the call.</par>

<par>As the name suggests, calls to <method>pushcolor</method> and <method>popcolor</method>
can be <z>nested</z>.</par>

<par>Note that the &ansi; escape sequences won't be written
to the stream immediately when calling <method>pushcolor</method>,
but only when output is written to the stream. This
means that the final call to <method>popcolor</method> in the above
example will not switch back to the default color.
To achieve this, call the method <method>finish</method> at the end
of all your write statements:</par>
<prog>
s.pushcolor(0x07)
s.write("spam")
s.pushcolor(0x0D)
s.write("eggs")
s.popcolor()
s.popcolor()
s.finish()
</prog>

<section><title>A simpler &api;</title>

<par>There is a second level &api; for <class>ansistyle.Stream</class>.
The class <class>ansistyle.Text</class> is a <z>colored text sequence</z>
and can be written to an <class>ansistyle.Stream</class> as well.</par>

<par>The color of a <class>Text</class> object is available via the class
attribute <lit>color</lit>, so derived classes can use their own color attribute.
ANSIStyle contains classes for all the 16 foreground colors named
<class>Text0</class>, ..., <class>Text15</class>.</par>

<par>In addition to the value <lit>0</lit>&ndash;<lit>127</lit>, <lit>-1</lit>
is allowed as a color value. This means <z>do not change the current color</z>.</par>

<par>A <class>Text</class> object is a sequence that may contain strings and
other <class>Text</class> objects and can be constructed simply by passing the
contained items to the constructor. So the above example can be written as:</par>
<prog>
s.write(ansistyle.Text7("spam", ansistyle.Text13("eggs")))
s.finish()
</prog>

<par><class>Text</class> objects have a <method>__str__</method> method that can be
used directly to print the object (the call to <method>finish</method> is included).
So the above example can be written as follows:</par>
<prog>
print ansistyle.Text7("spam", ansistyle.Text13("eggs"))
</prog>
</section>
"""


__version__ = tuple(map(int, "$Revision: 1.30 $"[11:-2].split(".")))
# $Source: /data/cvsroot/LivingLogic/Python/ansistyle/ansistyle.py,v $


import cStringIO

from _ansistyle import Stream


class Text(list):
	"""
	a colored string, colors can be numbers from <lit>0</lit> to <lit>15</lit>
	for the 16 ANSI colors or <lit>-1</lit> if this Text should use the current color.
	A <class>Text</class> object is a sequence, the sequence items may either be
	strings or <class>Text</class> objects themselves.
	"""
	color = -1 # the default color, which means "do not switch color"

	def __init__(self, *content):
		list.__init__(self, content)

	def append(self, *others):
		list.extend(self, others)

	def insert(self, index, *others):
		list.__setslice__(self, index, index, list(others))

	def __repr__(self):
		return "%s(%s)" % (self.__class__.__name__, list.__repr__(self)[1:-1])

	def __str__(self):
		"""
		return the resulting string with ANSI color escape sequences.
		"""
		stream = Stream(cStringIO.StringIO())
		stream.write(self) # might raise an exception if self contains other objects than string and Text objects
		stream.finish() # return to the default color
		return stream.stream.getvalue()


class Text0(Text):
	color = 0


class Text1(Text):
	color = 1


class Text2(Text):
	color = 2


class Text3(Text):
	color = 3


class Text4(Text):
	color = 4


class Text5(Text):
	color = 5


class Text6(Text):
	color = 6


class Text7(Text):
	color = 7


class Text8(Text):
	color = 8


class Text9(Text):
	color = 9


class Text10(Text):
	color = 10


class Text11(Text):
	color = 11


class Text12(Text):
	color = 12


class Text13(Text):
	color = 13


class Text14(Text):
	color = 14


class Text15(Text):
	color = 15


class EscapedText(Text):
	"""
	An extension to the <class>Text</class> class. Special characters can be
	replaced by customized <class>Text</class> objects via the <method>escapechar</method> method.
	"""
	def __init__(self, *data):
		newcontent = []
		for text in data:
			if isinstance(text, Text):
				newcontent.append(text)
			else:
				wasstr = None
				for c in text:
					c = self.escapechar(c)
					isstr = isinstance(c, basestring)
					if wasstr and isstr:
						newcontent[-1] += c
					else:
						newcontent.append(c)
					wasstr = isstr
		super(EscapedText, self).__init__(*newcontent)

	def escapechar(self, char):
		"""
		return a replacement <class>Text</class> object for the character
		<arg>char</arg> or <arg>char</arg> itself, if the character should be
		used as is. This method should be overwritten by subclasses.
		"""
		return char
